﻿Imports System
Imports System.Collections
Imports System.Configuration
Imports System.Data
Imports System.Linq
Imports System.Web
Imports System.Web.Security
Imports System.Web.UI
Imports System.Web.UI.HtmlControls
Imports System.Web.UI.WebControls
Imports System.Web.UI.WebControls.WebParts
Imports System.Xml.Linq

Partial Class ShowPoliciesAndClaimsLinqExercises
    Inherits System.Web.UI.Page

    ' we will be using the LINQ to SQL generated data context
    Private insuranceDB As DataClassesDataContext = New DataClassesDataContext()

    Protected Sub Page_Load(ByVal sender As Object, ByVal e As EventArgs)
        ' No error message to display 
        DatabaseError.Text = ""
    End Sub

    Protected Sub PoliciesView_SelectedIndexChanged(ByVal sender As Object, ByVal e As EventArgs)
        If PoliciesView.SelectedRow Is Nothing Then
            ' Don't dislay any edit/new claim dialog 
            ChangeClaimView.ActiveViewIndex = -1
        Else
            ' a policy has been selected by the user 
            ChangeClaimView.ActiveViewIndex = 1
        End If
    End Sub

    Protected Sub Button1_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles Button1.Click
        ' Only go ahead if validation has succeeded 
        If Page.IsValid() = True Then
            Dim polID As Integer = CInt(PoliciesView.SelectedValue)
            Dim polType = (From policy In insuranceDB.policies _
              Where policy.policyID = polID _
              Select policy.type).Single()

            Dim today As DateTime = System.DateTime.Today
            Dim claimDate As String = today.ToShortDateString()

            ' the LINQ query below was developed with the help of LINQpad 
            Dim refs As IEnumerable(Of Integer) = _
              From c In insuranceDB.claims() _
              Where c.reference.Substring(1, 4) = today.Year.ToString() _
              Select Integer.Parse(c.reference.Substring(9, 11))
            Dim nextRef As Integer = 0
            If refs.Count() > 0 Then
                nextRef = refs.Max() + 1
            End If

            ' A better format for these references would be
            ' (B|C)YYYY-RRRR, where YYYY is the year, and
            ' RRRRR is a four digit (say) reference number 
            Dim newRef As String = polType.Substring(0, 1).ToUpper() + today.Year.ToString() _
                + "-" + polID.ToString("99") + "-" + nextRef.ToString("000")
            Dim newClaim As claim = New claim()
            newClaim.claimDate = System.DateTime.Parse(claimDate)
            newClaim.reference = newRef
            newClaim.description = description.Text
            newClaim.amount = Convert.ToDecimal(amount.Text)
            newClaim.policyID = CInt(PoliciesView.SelectedValue)
            newClaim.approved = "N"

            Try
                DatabaseError.Text = ""
                insuranceDB.claims.InsertOnSubmit(newClaim)
                insuranceDB.SubmitChanges()
                ' refresh view of claims, and remove new claim dialog
                ClaimsView.DataBind()
                ChangeClaimView.ActiveViewIndex = -1
            Catch myEx As Exception
                DatabaseError.Text = "Database insertion failed." + myEx.Message
            End Try

        End If
    End Sub

    Protected Sub ClaimsView_SelectedIndexChanged(ByVal sender As Object, ByVal e As EventArgs)
        If ClaimsView.SelectedRow Is Nothing Then
            ChangeClaimView.ActiveViewIndex = -1
        Else
            ' show edit claim dialog
            ChangeClaimView.ActiveViewIndex = 0
        End If
    End Sub

    Protected Sub FormView1_ItemUpdated(ByVal sender As Object, ByVal e As FormViewUpdatedEventArgs)
        If e.Exception Is Nothing Then
            If e.AffectedRows = 1 Then
                DatabaseError.Text = ""
            Else
                DatabaseError.Text = "Rows affected: " + e.AffectedRows.ToString()
            End If
        Else
            DatabaseError.Text = "Database update failed. " + e.Exception.Message
        End If
        ClaimsView.DataBind()
        ChangeClaimView.ActiveViewIndex = -1
    End Sub

End Class

